@@ -78,9 +78,9 @@
                 "tools-for-build/SpecialCasing.txt"
                 "tools-for-build/EastAsianWidth.txt"
                 "tools-for-build/Scripts.txt"
-                "tools-for-build/LineBreakProperty.txt"
+                "tools-for-build/LineBreak.txt"
                 "tools-for-build/DerivedAge.txt"
-                "tools-for-build/Allkeys70.txt"
+                "tools-for-build/Allkeys80.txt"
                 "tools-for-build/ConfusablesEdited.txt"
                 "tools-for-build/BidiMirroring.txt"
                 "tools-for-build/Blocks.txt"
