@@ -50,11 +50,25 @@
 ;;; In the target SBCL, the INSTANCE type refers to a base
 ;;; implementation for compound types with lowtag
 ;;; INSTANCE-POINTER-LOWTAG. There's no way to express exactly that
-;;; concept portably, but we can get essentially the same effect by
-;;; testing for any of the standard types which would, in the target
-;;; SBCL, be derived from INSTANCE:
+;;; concept portably, but we know that anything derived from STRUCTURE!OBJECT
+;;; is equivalent to the target INSTANCE type. Also, because we use host packages
+;;; as proxies for target packages, those too must satisfy our INSTANCEP
+;;; - even if not a subtype of (OR STANDARD-OBJECT STRUCTURE-OBJECT).
+;;; Nothing else satisfies this definition of INSTANCEP.
+;;; As a guarantee that our set of host object types is exhaustive, we add one
+;;; more constraint when self-hosted: host instances of unknown type cause failure.
+;;; Some objects manipulated by the cross-compiler like the INTERVAL struct
+;;; - which is not a STRUCTURE!OBJECT - should never be seen as literals in code.
+;;; We assert that by way of the guard function.
+#+host-quirks-sbcl
+(defun unsatisfiable-instancep (x)
+  (when (and (host-sb-kernel:%instancep x)
+             (not (target-num-p x)))
+    (bug "%INSTANCEP test on ~S" x)))
 (deftype instance ()
-  '(or condition structure-object standard-object))
+  '(or structure!object package
+    #+host-quirks-sbcl (and host-sb-kernel:instance ; optimizes out a call when false
+                            (satisfies unsatisfiable-instancep))))
 (defun %instancep (x)
   (typep x 'instance))
 
@@ -93,7 +107,7 @@
   nil)
 
 (defun %negate (number)
-  (- number))
+  (sb-xc:- number))
 
 (defun %single-float (number)
   (coerce number 'single-float))
@@ -119,7 +133,11 @@
   (declare (ignore value))
   (error "cross-compiler can not make value cells"))
 
-;;; package locking nops for the cross-compiler
+;;; package-related stubs for the cross-compiler
+
+(defun find-undeleted-package-or-lose (string)
+  (or (find-package string)
+      (error "Cross-compiler bug: no package named ~S" string)))
 
 (defmacro without-package-locks (&body body)
   `(progn ,@body))
@@ -181,8 +199,6 @@
 (defun system-area-pointer-p (x) x nil) ; nothing is a SAP
 ;;; Needed for DEFINE-MOVE-FUN LOAD-SYSTEM-AREA-POINTER
 (defun sap-int (x) (error "can't take SAP-INT ~S" x))
-;;; Needed for FIXUP-CODE-OBJECT
-(defmacro without-gcing (&body body) `(progn ,@body))
 
 (defun logically-readonlyize (x) x)
 
@@ -237,7 +253,7 @@
 ;;; mainly for the sake of showing that it's quite easily done.
 ;;; Truth be told I'd have preferred to use the anti-expansion technique consistently,
 ;;; however occasionally we see things like (LET ((V FROB)) (%SVSET *THING* X V))
-;;; which means that the host is going to do the LET and the call %SVSET.
+;;; which means that the host is going to do the LET and then call %SVSET.
 (defun eval-tlf (form index &optional lexenv)
   (declare (ignore index lexenv))
   (flet ((matchp (template form &aux results)
@@ -246,6 +262,10 @@
                    (null (null form))
                    ((eql ?) (push form results) t) ; match and store anything
                    ((eql :ignore) t) ; match anything and disregard
+                   ((cons (eql :or))
+                    (some (lambda (template)
+                            (recurse form template))
+                          (cdr template)))
                    (cons (and (consp form)
                               (and (recurse (car form) (car template))
                                    (recurse (cdr form) (cdr template)))))
@@ -270,7 +290,7 @@
               `(defparameter ,symbol ,value)))
            (sb-impl::%defvar
             (destructuring-bind (symbol value) ; always occurs with a value
-                (matchp '((quote ?) (source-location) (unless (%boundp :ignore) ?))
+                (matchp '((quote ?) (source-location) (:or (unless (%boundp :ignore) ?) ?))
                         (cdr form))
               `(defvar ,symbol ,value)))
            (sb-c::%defconstant
