@@ -36,8 +36,8 @@
   NAME is some name (for debugging only) or NIL if there is no name."
   (declare (type function fun))
   (etypecase fun
-    #+sb-eval
-    (sb-eval:interpreted-function
+    (interpreted-function
+     #+sb-eval
      (let ((name (sb-eval:interpreted-function-name fun))
            (lambda-list (sb-eval:interpreted-function-lambda-list fun))
            (declarations (sb-eval:interpreted-function-declarations fun))
@@ -45,9 +45,8 @@
        (values `(lambda ,lambda-list
                   ,@(when declarations `((declare ,@declarations)))
                   ,@body)
-               t name)))
-    #+sb-fasteval
-    (sb-interpreter:interpreted-function
+               t name))
+     #+sb-fasteval
      (sb-interpreter:fun-lambda-expression fun))
     (function
      (let ((name (%fun-name fun))
@@ -181,8 +180,7 @@
   (typecase object
     (simple-fun "compiled function")
     (closure "compiled closure")
-    ((or #+sb-fasteval sb-interpreter:interpreted-function
-         #+sb-eval sb-eval:interpreted-function) "interpreted function")
+    (interpreted-function "interpreted function")
     (generic-function "generic-function")
     (t "funcallable-instance")))
 
@@ -398,6 +396,8 @@
                 (format stream "~@:_No subclasses.")))
           (unless (sb-mop:class-finalized-p class)
             (format stream "~@:_Not yet finalized."))
+          (when (eq :sealed (classoid-state (sb-pcl::class-classoid class)))
+            (format stream "~@:_Sealed."))
           (if (eq 'structure-class metaclass-name)
               (let* ((dd (find-defstruct-description name))
                      (slots (dd-slots dd)))
@@ -555,12 +555,9 @@
     (t
      (let ((source
              (typecase function
-               #+sb-eval
-               (sb-eval:interpreted-function
-                (sb-eval:interpreted-function-source-location function))
-               #+sb-fasteval
-               (sb-interpreter:interpreted-function
-                (sb-interpreter:fun-source-location function)))))
+               (interpreted-function
+                #+sb-eval (sb-eval:interpreted-function-source-location function)
+                #+sb-fasteval (sb-interpreter:fun-source-location function)))))
        (when source
          (let ((namestring (sb-c:definition-source-location-namestring source)))
            (when namestring
