@@ -46,7 +46,7 @@
 ;;; fail to infer (< I FIXNUM): it does not understand that this
 ;;; constraint follows from (TYPEP I (INTEGER 0 0)).
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 ;;; *CONSTRAINT-UNIVERSE* gets bound in IR1-PHASES to a fresh,
 ;;; zero-length, non-zero-total-size vector-with-fill-pointer.
@@ -72,7 +72,14 @@
   ;; EQL
   ;;     X is a LAMBDA-VAR and Y is a LVAR, a LAMBDA-VAR or a CONSTANT.
   ;;     The relation is asserted to hold.
-  (kind nil :type (member typep < > eql))
+  ;;
+  ;; ARRAY-IN-BOUNDS-P
+  ;;     To handle (array-in-bounds-p array var) and
+  ;;     (array-in-bounds-p array 10) X can be either the lambda-var
+  ;;     of ARRAY or VAR, while Y is either the lambda-var of VAR or a
+  ;;     constant.
+  (kind nil :type (member typep < > eql
+                          array-in-bounds-p))
   ;; The operands to the relation.
   (x nil :type lambda-var)
   (y nil :type constraint-y)
@@ -143,8 +150,7 @@
               (:constructor make-conset ())
               (:copier %copy-conset))
     (vector (make-array
-             ;; FIXME: make POWER-OF-TWO-CEILING available earlier?
-             (ash 1 (integer-length (1- (length *constraint-universe*))))
+             (power-of-two-ceiling (length *constraint-universe*))
              :element-type 'bit :initial-element 0)
             :type simple-bit-vector)
     ;; Bit-vectors win over lightweight hashes for copy, union,
@@ -157,9 +163,6 @@
     (or (= (conset-min conset) (conset-max conset))
         (not (find 1 (conset-vector conset)
                    :start (conset-min conset)
-                   ;; the :end argument can be commented out when
-                   ;; bootstrapping on a < 1.0.9 SBCL errors out with
-                   ;; a full call to DATA-VECTOR-REF-WITH-OFFSET.
                    :end (conset-max conset)))))
 
   (defun copy-conset (conset)
@@ -171,8 +174,8 @@
     (declare (type index new-size))
     (setf (conset-vector conset)
           (replace (the simple-bit-vector
-                     (make-array
-                      (ash 1 (integer-length (1- new-size)))
+                      (make-array
+                       (power-of-two-ceiling new-size)
                       :element-type 'bit
                       :initial-element 0))
                    (the simple-bit-vector
@@ -294,7 +297,7 @@
   (etypecase y
     (ctype
        (awhen (lambda-var-ctype-constraints x)
-         (dolist (con (gethash (sb!kernel::type-class-info y) it) nil)
+         (dolist (con (gethash (sb-kernel::type-class-info y) it) nil)
            (when (and (eq (constraint-kind con) kind)
                       (eq (constraint-not-p con) not-p)
                       (type= (constraint-y con) y))
@@ -332,14 +335,14 @@
   (macrolet ((ensuref (place default)
                `(or ,place (setf ,place ,default)))
              (ensure-hash (place)
-               `(ensuref ,place (make-hash-table)))
+               `(ensuref ,place (make-hash-table :test #'eq)))
              (ensure-vec (place)
                `(ensuref ,place (make-array 8 :adjustable t :fill-pointer 0))))
     (etypecase y
       (ctype
        (let ((index (ensure-hash (lambda-var-ctype-constraints x)))
              (vec   (ensure-vec  (lambda-var-inheritable-constraints x))))
-         (push con (gethash (sb!kernel::type-class-info y) index))
+         (push con (gethash (sb-kernel::type-class-info y) index))
          (vector-push-extend con vec)))
       (lvar
        (let ((index (ensure-hash (lambda-var-eq-constraints x))))
@@ -355,7 +358,8 @@
           (let ((vec (ensure-vec (lambda-var-private-constraints x))))
             (vector-push-extend con vec)))
          (lambda-var
-          (let ((vec (if (constraint-not-p con)
+          (let ((vec (if (or (constraint-not-p con)
+                             (eq (constraint-kind con) 'array-in-bounds-p))
                          (ensure-vec (lambda-var-inheritable-constraints x))
                          (ensure-vec (lambda-var-eql-var-constraints x)))))
             (vector-push-extend con vec)))))))
@@ -511,11 +515,14 @@
     (let ((eq-x (eq from-var (constraint-x con)))
           (eq-y (eq from-var (constraint-y con))))
       (dolist (var vars)
-        (conset-add-constraint target
-                               (constraint-kind con)
-                               (if eq-x var (constraint-x con))
-                               (if eq-y var (constraint-y con))
-                               (constraint-not-p con))))))
+        (let ((x (if eq-x var (constraint-x con)))
+              (y (if eq-y var (constraint-y con))))
+          (unless (eq x y)
+            (conset-add-constraint target
+                                   (constraint-kind con)
+                                   x
+                                   y
+                                   (constraint-not-p con))))))))
 
 ;; Add an (EQL LAMBDA-VAR LAMBDA-VAR) constraint on VAR1 and VAR2 and
 ;; inherit each other's constraints.
@@ -535,7 +542,7 @@
 
 ;;; If REF is to a LAMBDA-VAR with CONSTRAINTs (i.e. we can do flow
 ;;; analysis on it), then return the LAMBDA-VAR, otherwise NIL.
-#!-sb-fluid (declaim (inline ok-ref-lambda-var))
+#-sb-fluid (declaim (inline ok-ref-lambda-var))
 (defun ok-ref-lambda-var (ref)
   (declare (type ref ref))
   (let ((leaf (ref-leaf ref)))
@@ -558,6 +565,7 @@
 ;;;; Searching constraints
 
 ;;; Add the indicated test constraint to TARGET.
+(declaim (inline precise-add-test-constraint))
 (defun precise-add-test-constraint (fun x y not-p constraints target)
   (if (and (eq 'eql fun) (lambda-var-p y) (not not-p))
       (add-eql-var-var-constraint x y constraints target)
@@ -571,7 +579,7 @@
          (precise-add-test-constraint fun x y not-p constraints target))))
 ;;; Add complementary constraints to the consequent and alternative
 ;;; blocks of IF. We do nothing if X is NIL.
-(declaim (inline precise-add-test-constraint quick-add-complement-constraints))
+(declaim (inline quick-add-complement-constraints))
 (defun precise-add-complement-constraints (fun x y not-p constraints
                                            consequent-constraints
                                            alternative-constraints)
@@ -643,6 +651,30 @@
                                         constraints)
              ptype nil)))))
 
+(defun array-in-bounds-p-constraints (constraints index-lvar index-var
+                                      length-lvar)
+  (let ((index-constant
+          (and (not index-var)
+               (let ((use (principal-lvar-use index-lvar)))
+                 (and (ref-p use)
+                      (constant-p (ref-leaf use))
+                      (ref-leaf use)))))
+        (array-lvar
+          (let ((use (principal-lvar-ref-use length-lvar)))
+            (and (combination-p use)
+                 (lvar-fun-is (combination-fun use)
+                              '(vector-length))
+                 (car (combination-args use))))))
+    (when (and (or index-var index-constant)
+               array-lvar)
+      (let ((array-var (ok-lvar-lambda-var array-lvar constraints)))
+        (when array-var
+          (if index-constant
+              ;; Attach the constaraint to the array if
+              ;; the index is constant
+              (values 'array-in-bounds-p array-var index-constant)
+              (values 'array-in-bounds-p index-var array-var)))))))
+
 ;;; Add test constraints to the consequent and alternative blocks of
 ;;; the test represented by USE.
 (defun add-test-constraints (use if constraints)
@@ -668,8 +700,9 @@
           (combination
            (unless (eq (combination-kind use)
                        :error)
-             (let ((name (lvar-fun-name
-                          (basic-combination-fun use)))
+             (let ((name (uncross
+                          (lvar-fun-name
+                           (basic-combination-fun use))))
                    (args (basic-combination-args use)))
                (case name
                  ((%typep %instance-typep)
@@ -713,14 +746,29 @@
                                                 nil constraints
                                                 consequent-constraints)))))
                  ((< >)
-                  (let* ((arg1 (first args))
-                         (var1 (ok-lvar-lambda-var arg1 constraints))
-                         (arg2 (second args))
-                         (var2 (ok-lvar-lambda-var arg2 constraints)))
-                    (when var1
-                      (add name var1 (lvar-type arg2) nil))
-                    (when var2
-                      (add (if (eq name '<) '> '<) var2 (lvar-type arg1) nil))))
+                  (when (= (length args) 2)
+                    (flet ((handle-array-in-bounds-p (index-arg index-var length-arg)
+                             (multiple-value-bind (kind x y)
+                                 (array-in-bounds-p-constraints constraints index-arg index-var
+                                                                length-arg)
+                               (when kind
+                                 (add-test-constraint quick-p
+                                                      kind x y
+                                                      nil constraints
+                                                      consequent-constraints)))))
+                      (let* ((arg1 (first args))
+                             (var1 (ok-lvar-lambda-var arg1 constraints))
+                             (arg2 (second args))
+                             (var2 (ok-lvar-lambda-var arg2 constraints)))
+                        (case name
+                          (<
+                           (handle-array-in-bounds-p arg1 var1 arg2))
+                          (>
+                           (handle-array-in-bounds-p arg2 var2 arg1)))
+                        (when var1
+                          (add name var1 (lvar-type arg2) nil))
+                        (when var2
+                          (add (if (eq name '<) '> '<) var2 (lvar-type arg1) nil))))))
                  (t
                   (add-combination-test-constraints use constraints
                                                     consequent-constraints
@@ -877,6 +925,10 @@
                      (mapc-member-type-members #'note-not other)
                      (setq not-res (type-union not-res other)))
                  (setq res (type-approx-intersection2 res other))))
+            (array-in-bounds-p
+             (unless (eq (ref-constraints ref)
+                         (pushnew con (ref-constraints ref)))
+               (reoptimize-lvar (node-lvar ref))))
             (eql
              (let ((other-type (leaf-type other)))
                (if not-p
@@ -915,13 +967,26 @@
            (setf (node-derived-type ref) *wild-type*)
            (change-ref-leaf ref (find-constant t)))
           (t
-           (setf not-res
-                 (type-union not-res (make-member-type :xset not-set :fp-zeroes not-fpz)))
-           (derive-node-type ref
-                             (make-single-value-type
-                              (or (type-difference res not-res)
-                                  res)))
-           (maybe-terminate-block ref nil))))
+           (let ((type (type-difference res
+                                        (type-union not-res
+                                                    (make-member-type not-set not-fpz)))))
+             ;; CHANGE-CLASS can change the type, lower down to standard-object,
+             ;; type propagation for classes is not as important anyway.
+             (cond #-sb-xc-host
+                   ((and
+                     (eq sb-pcl::**boot-state** 'sb-pcl::complete)
+                     (block nil
+                       (let ((standard-object (find-classoid 'standard-object)))
+                         (sb-kernel::map-type
+                          (lambda (type)
+                            (when (and (classoid-p type)
+                                       (csubtypep type standard-object))
+                              (return t)))
+                          type)))))
+                   (t
+                    (derive-node-type ref
+                                      (make-single-value-type type))
+                    (maybe-terminate-block ref nil)))))))
   (values))
 
 ;;;; Flow analysis
@@ -950,7 +1015,7 @@
                           conset)
                 constraint-propagate-in-block))
 (defun constraint-propagate-in-block (block gen preprocess-refs-p)
-  (do-nodes (node lvar block)
+  (do-nodes (node nil block)
     (typecase node
       (bind
        (let ((fun (bind-lambda node)))
@@ -969,19 +1034,31 @@
          (when preprocess-refs-p
            (constrain-ref-type node gen))))
       (cast
-       (let ((lvar (cast-value node)))
-         (let ((var (ok-lvar-lambda-var lvar gen)))
-           (when var
-             (let ((atype (single-value-type (cast-derived-type node)))) ;FIXME
-               (unless (eq atype *universal-type*)
-                 (conset-add-constraint-to-eql gen 'typep var atype nil)))))))
+
+       (let* ((lvar (cast-value node))
+              (var (ok-lvar-lambda-var lvar gen)))
+         (when var
+           (let ((atype (single-value-type (cast-derived-type node)))) ;FIXME
+             (unless (eq atype *universal-type*)
+               (conset-add-constraint-to-eql gen 'typep var atype nil))))
+         (when (and (bound-cast-p node)
+                    (bound-cast-check node)
+                    (not (node-deleted (bound-cast-check node))))
+           (let ((check-bound (bound-cast-check node)))
+             (destructuring-bind (array dim index)
+                 (combination-args check-bound)
+               (declare (ignore array))
+               (multiple-value-bind (kind x y)
+                   (array-in-bounds-p-constraints gen index var dim)
+                 (when kind
+                   (conset-add-constraint-to-eql gen kind x y nil))))))))
       (cset
        (binding* ((var (set-var node))
                   (nil (lambda-var-p var) :exit-if-null)
                   (nil (lambda-var-constraints var) :exit-if-null))
          (when (policy node (and (= speed 3) (> speed compilation-speed)))
            (let ((type (lambda-var-type var)))
-             (unless (eql *universal-type* type)
+             (unless (eq *universal-type* type)
                (do-eql-vars (other (var gen))
                  (unless (eql other var)
                    (conset-add-constraint gen 'typep other type nil))))))
@@ -1029,7 +1106,6 @@
                   (block-in block)
                   (copy-conset (block-in block)))
               final-pass-p)))
-    (setf (block-gen block) gen)
     (multiple-value-bind (consequent-constraints alternative-constraints)
         (constraint-propagate-if block gen)
       (if consequent-constraints
@@ -1154,11 +1230,12 @@
           (rest-of-blocks ())
           (seen-loop-p ()))
       (do-blocks (block component)
-        (when (and (not seen-loop-p) (loopy-p block))
-          (setq seen-loop-p t))
-        (if seen-loop-p
-            (push block rest-of-blocks)
-            (push block leading-blocks)))
+        (when (block-type-check block)
+          (when (and (not seen-loop-p) (loopy-p block))
+            (setq seen-loop-p t))
+          (if seen-loop-p
+              (push block rest-of-blocks)
+              (push block leading-blocks))))
       (values (nreverse leading-blocks) (nreverse rest-of-blocks)))))
 
 ;;; Append OBJ to the end of LIST as if by NCONC but only if it is not
@@ -1178,7 +1255,8 @@
   (let ((blocks-to-process ()))
     (flet ((enqueue (blocks)
              (dolist (block blocks)
-               (setq blocks-to-process (nconc-new block blocks-to-process)))))
+               (when (block-type-check block)
+                 (setq blocks-to-process (nconc-new block blocks-to-process))))))
       (multiple-value-bind (leading-blocks rest-of-blocks)
           (leading-component-blocks component)
         ;; Update every block once to account for changes in the
@@ -1205,10 +1283,14 @@
 (defun constraint-propagate (component)
   (declare (type component component))
   (init-var-constraints component)
-
-  (unless (block-out (component-head component))
-    (setf (block-out (component-head component)) (make-conset)))
-
+  ;; Previous results can confuse propagation and may loop forever
+  (do-blocks (block component)
+    (setf (block-out block) nil)
+    (let ((last (block-last block)))
+      (when (if-p last)
+        (setf (if-alternative-constraints last) nil)
+        (setf (if-consequent-constraints last) nil))))
+  (setf (block-out (component-head component)) (make-conset))
   (dolist (block (find-and-propagate-constraints component))
     (unless (block-delete-p block)
       (use-result-constraints block)))
