@@ -59,8 +59,7 @@
 (defun instrument-alloc (size node)
   (when (policy node (> sb-c::instrument-consing 1))
     (let ((skip-instrumentation (gen-label)))
-      (inst mov temp-reg-tn
-            (ea (* n-word-bytes thread-profile-data-slot) thread-base-tn))
+      (inst mov temp-reg-tn (thread-slot-ea thread-profile-data-slot))
       (inst test temp-reg-tn temp-reg-tn)
       ;; This instruction is modified to "JMP :z" when profiling is
       ;; partially enabled. After the buffer is assigned, it becomes
@@ -172,9 +171,9 @@
 ;;; Allocate an other-pointer object of fixed SIZE with a single word
 ;;; header having the specified WIDETAG value. The result is placed in
 ;;; RESULT-TN.
-(defun fixed-alloc (result-tn widetag size node &optional stack-allocate-p
+(defun alloc-other (result-tn widetag size node &optional stack-allocate-p
                     &aux (bytes (pad-data-block size)))
-  (let ((header (logior (ash (1- size) n-widetag-bits) widetag)))
+  (let ((header (compute-object-header size widetag)))
     (cond (stack-allocate-p
            (allocation result-tn bytes node t other-pointer-lowtag)
            (storew header result-tn 0 other-pointer-lowtag))
@@ -358,7 +357,7 @@
         ;; It would also be good to skip zero-fill of specialized vectors
         ;; perhaps in a policy-dependent way. At worst you'd see random
         ;; bits, and CLHS says consequences are undefined.
-        (when sb-c::*msan-unpoison*
+        (when (sb-c:msan-unpoison sb-c:*compilation*)
           ;; Unpoison all DX vectors regardless of widetag.
           ;; Mark the header and length as valid, not just the payload.
           #+linux ; unimplemented for others
@@ -504,7 +503,7 @@
   (:results (result :scs (descriptor-reg) :from :argument))
   (:node-var node)
   (:generator 37
-    (fixed-alloc result fdefn-widetag fdefn-size node)
+    (alloc-other result fdefn-widetag fdefn-size node)
     (storew name result fdefn-name-slot other-pointer-lowtag)
     (storew nil-value result fdefn-fun-slot other-pointer-lowtag)
     (storew (make-fixup 'undefined-tramp :assembly-routine)
@@ -533,7 +532,7 @@
                        temp)
                 result 0 fun-pointer-lowtag (not stack-allocate-p)))
      ;; Done with pseudo-atomic
-     (inst lea temp (rip-relative-ea label (ash simple-fun-code-offset word-shift)))
+     (inst lea temp (rip-relative-ea label (ash simple-fun-insts-offset word-shift)))
      (storew temp result closure-fun-slot fun-pointer-lowtag))))
 
 ;;; The compiler likes to be able to directly make value cells.
@@ -543,7 +542,7 @@
   (:info stack-allocate-p)
   (:node-var node)
   (:generator 10
-    (fixed-alloc result value-cell-widetag value-cell-size node stack-allocate-p)
+    (alloc-other result value-cell-widetag value-cell-size node stack-allocate-p)
     (storew value result value-cell-value-slot other-pointer-lowtag)))
 
 ;;;; automatic allocators for primitive objects
@@ -599,7 +598,6 @@
                 (inst or :byte result lowtag))))))
     (when instancep ; store its layout
       (inst mov :dword (ea (+ 4 (- lowtag)) result)
-            ;; XXX: should layout fixups use a name, not a layout object?
             (make-fixup type :layout))))))
 
 ;;; Allocate a non-vector variable-length object.
