@@ -70,12 +70,6 @@ os_vm_size_t os_vm_page_size;
 #include "gc.h"
 #include "gencgc-internal.h"
 #include <wincrypt.h>
-
-#if 0
-int linux_sparc_siginfo_bug = 0;
-int linux_supports_futex=0;
-#endif
-
 #include <stdarg.h>
 #include <string.h>
 
@@ -401,13 +395,13 @@ void unmap_gc_page()
  * to leave old-style linking code in place for the sake of
  * _non-linkage-table_ platforms (they probably don't have -ldl or its
  * equivalent, like LL/GPA, at all) -- but i did it usually by moving
- * the entire `old style' code under #!-sb-dynamic-core and
+ * the entire `old style' code under #-sb-dynamic-core and
  * refactoring the `new style' branch, instead of cutting the tail
- * piecemeal and increasing #!+-ifdeffery amount & the world enthropy.
+ * piecemeal and increasing #+-ifdeffery amount & the world enthropy.
  *
  * If we look at the majority of the ``new style'' code units, it's a
- * common thing to observe how #!+-ifdeffery _vanishes_ instead of
- * multiplying: #!-sb-xc, #!+sb-xc-host and #!-sb-xc-host end up
+ * common thing to observe how #+-ifdeffery _vanishes_ instead of
+ * multiplying: #-sb-xc, #+sb-xc-host and #-sb-xc-host end up
  * needing the same code. Runtime checks of static v. dynamic symbol
  * disappear even faster. STDCALL mangling and leading underscores go
  * out of scope (and GCed, hopefully) instead of surfacing here and
@@ -653,7 +647,7 @@ void* os_dlsym_default(char* name)
    started before _any_ TLS slot is allocated by libraries, and
    some C compiler vendors rely on this fact. */
 
-void os_preinit()
+int os_preinit(char *argv[], char *envp[])
 {
 #ifdef LISP_FEATURE_X86
     DWORD slots[TLS_MINIMUM_AVAILABLE];
@@ -680,6 +674,7 @@ void os_preinit()
              "(last TlsAlloc() returned %u)",key);
     }
 #endif
+    return 0;
 }
 #endif  /* LISP_FEATURE_SB_THREAD */
 
@@ -1368,9 +1363,17 @@ handle_exception(EXCEPTION_RECORD *exception_record,
         return ExceptionContinueSearch;
     }
 
+    DWORD code = exception_record->ExceptionCode;
+
+    if(code == 0x20474343 || /* GCC */
+       code == 0xE06D7363 || /* Emsc */
+       code == 0xE0434352)   /* ECCR */
+        /* Do not handle G++, VC++ and .NET exceptions */
+        return ExceptionContinueSearch;
+
     DWORD lastError = GetLastError();
     DWORD lastErrno = errno;
-    DWORD code = exception_record->ExceptionCode;
+
     struct thread* self = arch_os_get_current_thread();
 
     os_context_t context, *ctx = &context;
@@ -1910,6 +1913,7 @@ win32_unix_write(HANDLE handle, void * buf, int count)
     LARGE_INTEGER file_position;
     BOOL seekable;
     BOOL ok;
+    DWORD errorCode;
 
     if (console_handle_p(handle)) {
         return win32_write_console(handle,buf,count);
@@ -1930,19 +1934,20 @@ win32_unix_write(HANDLE handle, void * buf, int count)
 
     WITH_INTERRUPTIBLE_IO(handle) {
         ok = WriteFile(handle, buf, count, &written_bytes, &overlapped);
+        if (!ok)
+            errorCode = GetLastError();
     }
 
     if (ok) {
         goto done_something;
     } else {
-        DWORD errorCode = GetLastError();
         if (errorCode==ERROR_OPERATION_ABORTED) {
             GetOverlappedResult(handle,&overlapped,&written_bytes,FALSE);
             errno = EINTR;
             return -1;
         }
         if (errorCode!=ERROR_IO_PENDING) {
-            errno = EIO;
+            errno = errorCode;
             return -1;
         } else {
             if(WaitForMultipleObjects(2,self->private_events.events,
@@ -1953,10 +1958,10 @@ win32_unix_write(HANDLE handle, void * buf, int count)
                 waitInGOR = FALSE;
             }
             if (!GetOverlappedResult(handle,&overlapped,&written_bytes,waitInGOR)) {
-                if (GetLastError()==ERROR_OPERATION_ABORTED) {
+                if ((errorCode = GetLastError())==ERROR_OPERATION_ABORTED) {
                     errno = EINTR;
                 } else {
-                    errno = EIO;
+                    errno = errorCode;
                 }
                 return -1;
             } else {
@@ -2005,13 +2010,14 @@ win32_unix_read(HANDLE handle, void * buf, int count)
 
     WITH_INTERRUPTIBLE_IO(handle) {
         ok = ReadFile(handle,buf,count,&read_bytes, &overlapped);
+        if (!ok)
+            errorCode = GetLastError();
     }
 
     if (ok) {
         /* immediately */
         goto done_something;
     } else {
-        errorCode = GetLastError();
         if (errorCode == ERROR_HANDLE_EOF ||
             errorCode == ERROR_BROKEN_PIPE ||
             errorCode == ERROR_NETNAME_DELETED) {
