@@ -26,6 +26,7 @@
 #include "pseudo-atomic.h"
 #include "forwarding-ptr.h"
 #include "var-io.h"
+#include "code.h"
 
 #include "genesis/static-symbols.h"
 #include "genesis/symbol.h"
@@ -40,9 +41,6 @@
 #define BREAKPOINT_WIDTH 2
 #endif
 
-void arch_init(void)
-{}
-
 #ifndef LISP_FEATURE_WIN32
 os_vm_address_t
 arch_get_bad_addr(int sig, siginfo_t *code, os_context_t *context)
@@ -62,13 +60,15 @@ arch_get_bad_addr(int sig, siginfo_t *code, os_context_t *context)
 int *
 context_eflags_addr(os_context_t *context)
 {
-#if defined __linux__ || defined __sun
+#if defined __linux__
     /* KLUDGE: As of kernel 2.2.14 on Red Hat 6.2, there's code in the
      * <sys/ucontext.h> file to define symbolic names for offsets into
      * gregs[], but it's conditional on __USE_GNU and not defined, so
      * we need to do this nasty absolute index magic number thing
      * instead. */
     return &context->uc_mcontext.gregs[16];
+#elif defined(LISP_FEATURE_SUNOS)
+    return &context->uc_mcontext.gregs[EFL];
 #elif defined(LISP_FEATURE_FREEBSD) || defined(__DragonFly__)
     return &context->uc_mcontext.mc_eflags;
 #elif defined __OpenBSD__
@@ -348,8 +348,7 @@ arch_install_interrupt_handlers()
 void
 gencgc_apply_code_fixups(struct code *old_code, struct code *new_code)
 {
-    char* code_start_addr = (char*)(code_header_words(new_code->header)
-                                    + (lispobj*)new_code);
+    char* code_start_addr = code_text_start(new_code);
     os_vm_size_t displacement = (char*)new_code - (char*)old_code;
     lispobj fixups = new_code->fixups;
     /* It will be a nonzero integer if valid, or 0 if there are no fixups */
