@@ -9,7 +9,7 @@
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
 
-#-(and x86-64 immobile-space sb-thread) (sb-ext:exit :code 104) ; can't run these tests
+#-x86-64 (sb-ext:exit :code 104)
 
 (defun disasm (safety expr &optional (remove-epilogue t))
   ;; This lambda has a name because if it doesn't, then the name
@@ -23,22 +23,17 @@
     (sb-int:encapsulate 'sb-disassem::add-debugging-hooks 'test
                         (lambda (f &rest args) (declare (ignore f args))))
     (let ((lines
-           (split-string
-            (with-output-to-string (s)
-              (let ((sb-disassem:*disassem-location-column-width* 0))
-                (disassemble fun :stream s)))
-            #\newline)))
+            (split-string
+             (with-output-to-string (s)
+               (let ((sb-disassem:*disassem-location-column-width* 0))
+                 (disassemble fun :stream s)))
+             #\newline)))
       (sb-int:unencapsulate 'sb-disassem::add-debugging-hooks 'test)
-      (setq lines (cddr lines)) ; remove "Disassembly for"
+      (setq lines (cddr lines))         ; remove "Disassembly for"
       ;; For human-readability, kill the whitespace
       (setq lines (mapcar (lambda (x) (string-left-trim " ;" x)) lines))
-      ;; Same old same old- filler in simple-funs manifests as one of two
-      ;; things dependin on whether even-or odd-length padding was needed.
-      (loop while (member (car (last lines))
-                          '("00               BYTE #X00"
-                            "0000             ADD [RAX], AL"
-                            "") :test 'string=)
-            do (setq lines (nbutlast lines)))
+      (when (string= (car (last lines)) "")
+        (setq lines (nbutlast lines)))
       ;; Remove safepoint traps
       (setq lines (remove-if (lambda (x) (search "; safepoint" x)) lines))
       ;; If the last 4 lines are of the expected form
@@ -51,7 +46,8 @@
           (butlast lines 4)
           lines))))
 
-(with-test (:name :symeval-known-thread-local)
+(with-test (:name :symeval-known-thread-local
+            :skipped-on (not :sb-thread))
   ;; It should take 1 instruction to read a known thread-local var
   (assert (= (length (disasm 1 'sb-thread:*current-thread*)) 1))
   (assert (= (length (disasm 1 'sb-sys:*interrupt-pending*)) 1))
@@ -66,7 +62,9 @@
 ;; DO NOT REMOVE!
 (compile nil '(lambda (foo) (eval 'frob)))
 
-(with-test (:name :symeval-known-tls-index :skipped-on :immobile-symbols)
+(with-test (:name :symeval-known-tls-index
+            :skipped-on (or (not :immobile-space) :immobile-symbols
+                            (not :sb-thread)))
   ;; When symbol SC is IMMEDIATE:
   ;;    498B9578210000     MOV RDX, [R13+disp]       ; tls: *PRINT-BASE*
   ;;    83FA61             CMP EDX, 97
@@ -83,10 +81,12 @@
 
 (defvar *blub*) ; immobile space
 (defvar blub)   ; dynamic space
-(assert (sb-kernel:immobile-space-obj-p '*blub*))
+#+immobile-space (assert (sb-kernel:immobile-space-obj-p '*blub*))
 #-immobile-symbols (assert (not (sb-kernel:immobile-space-obj-p 'blub)))
 
-(with-test (:name :symeval-unknown-tls-index :skipped-on :immobile-symbols)
+(with-test (:name :symeval-unknown-tls-index
+            :skipped-on (or (not :immobile-space) :immobile-symbols
+                            (not :sb-thread)))
   ;; When symbol SC is immediate:
   ;;    8B142514A24C20     MOV EDX, [#x204CA214]    ; tls_index: *BLUB*
   ;;    4A8B142A           MOV RDX, [RDX+R13]
@@ -120,9 +120,7 @@
 #+immobile-code
 (with-test (:name :reference-assembly-tramp)
   (dolist (testcase '(("FUNCALLABLE-INSTANCE-TRAMP"
-                       sb-kernel:%make-funcallable-instance)
-                      ("UNDEFINED-FDEFN"
-                       sb-kernel:make-fdefn)))
+                       sb-kernel:%make-funcallable-instance)))
     (let ((lines
            (split-string
             (with-output-to-string (stream)
@@ -135,6 +133,23 @@
                                  ;; and verify disassembly
                                  (search (car testcase) line)))))))
 
+#+immobile-code ; uses SB-C::*COMPILE-TO-MEMORY-SPACE*
+(with-test (:name :static-link-compile-to-memory)
+  (let* ((string
+          (with-output-to-string (stream)
+            (disassemble
+             (let ((sb-c::*compile-to-memory-space* :immobile))
+               (compile nil '(lambda () (print (gensym)))))
+             :stream stream)))
+         (lines (split-string string #\newline)))
+    (flet ((find-line (mnemonic operand)
+             (find-if (lambda (line)
+                        (and (search mnemonic line)
+                             (search operand line)))
+                      lines)))
+    (assert (find-line "CALL" "FUNCTION GENSYM"))
+    (assert (find-line "JMP" "FUNCTION PRINT")))))
+
 #+immobile-code
 (with-test (:name :static-unlinker)
   (let ((sb-c::*compile-to-memory-space* :immobile))
@@ -168,7 +183,7 @@
         (expect "#<FDEFN G>" lines)))))
 
 (with-test (:name :c-call
-            :broken-on (not :sb-dynamic-core))
+            :skipped-on (not :sb-dynamic-core))
   (let* ((lines (split-string
                  (with-output-to-string (s)
                    (let ((sb-disassem:*disassem-location-column-width* 0))
@@ -193,7 +208,8 @@
     (assert success)))
 
 (defglobal *avar* nil)
-(with-test (:name :set-symbol-value-imm-2)
+(with-test (:name :set-symbol-value-imm-2
+                  :skipped-on (not :immobile-space))
   (let (success)
     (dolist (line (split-string
                    (with-output-to-string (s)
@@ -263,7 +279,8 @@
         (setq success t)))
     (assert success)))
 
-(with-test (:name :list-vop-immediate-to-mem)
+(with-test (:name :list-vop-immediate-to-mem
+                  :skipped-on (not :immobile-space))
   (let ((lines
          (split-string
           (with-output-to-string (s)
@@ -273,3 +290,42 @@
     (assert (loop for line in lines
                   thereis (and (search "MOV QWORD PTR [" line)
                                (search ":KEY" line))))))
+
+(defstruct thing x)
+(with-test (:name :instance-ref-eq
+                  :skipped-on (not :immobile-space))
+  (let ((lines
+         (split-string
+          (with-output-to-string (s)
+            (disassemble '(lambda (obj)
+                           (if (eq (thing-x (truly-the thing obj)) :yup) 1 2))
+                         :stream s))
+          #\newline)))
+    ;; assert that the comparison instruction performed the memory load
+    (assert (loop for line in lines
+                  thereis
+                  (and (search "CMP QWORD PTR [" line)
+                       (search ":YUP" line))))))
+
+(with-test (:name :fixnump-thing-ref)
+  (flet ((try (access-form true false)
+           (let* ((f (compile nil `(lambda (obj) (typep ,access-form 'fixnum))))
+                  (lines
+                   (split-string
+                    (with-output-to-string (s) (disassemble f :stream s))
+                    #\newline)))
+             (assert (funcall f true))
+             (assert (not (funcall f false)))
+             ;; assert that the TEST instruction dereferenced OBJ and performed the test
+             (assert (loop for line in lines
+                           thereis
+                           (and (search "TEST BYTE PTR [" line)
+                                (search (format nil ", ~D" sb-vm:fixnum-tag-mask)
+                                        line)))))))
+    (try '(thing-x (truly-the thing obj))
+         (make-thing :x 1) (make-thing :x "hi"))
+    (try '(car obj) '(1) '("hi"))
+    (try '(cdr obj) '("hi" . 1) '("hi"))))
+
+(with-test (:name :huge-code :skipped-on (not :immobile-code))
+  (sb-vm::allocate-code-object :immobile 4 (* 2 1024 1024)))
